/* Name: hwconfig.h
 * Project: HID-LightCtrl
 * Author: SFR frydaysoft.d
 * Creation Date: 2013-03-19
 * Tabsize: 4
 * Note: basics used from OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt)
 * This Revision: $Id: bootloaderconfig.h 699 2008-11-26 19:50:32Z cs $
 */

#ifndef __hwconfig_h_included__
#define __hwconfig_h_included__

#define HID_LIGHT_HW
#define LOW_COST_HW_WITH_ULN2003
 
/*
General Description:
This file (together with some settings in Makefile) configures the HID-LightCtrl
according to the hardware.

This file contains (besides the hardware configuration normally found in
usbconfig.h) two functions or macros: HwLowInit() and
AppCondition(). Whether you implement them as macros or as static
inline functions is up to you, decide based on code size and convenience.

HwInit() is called as one of the first actions after reset. It should
be a minimum initialization of the hardware so that the boot loader condition
can be read. This will usually consist of activating a pull-up resistor for an
external jumper which selects boot loader mode. You may call leaveBootloader()
from this function if you know that the main code should run.

AppCondition() is called immediately after initialization and in each
main loop iteration. If it returns TRUE, the application will be active. If it
returns FALSE, the boot loader jumps to address 0 (the loaded application)
immediately.

*/

/* ---------------------------- Hardware Config ---------------------------- */

#ifdef HID_LIGHT_HW 

  // *********************************************************************************
  #ifdef LOW_COST_HW_WITH_ULN2003
    // ***** Atmega8 with ULN2003
    // PC0-PC3, PD4-PD7
    #define InitOutpPort()    PORTC &= 0xF0; DDRC |= 0x0F; PORTD &= 0x0F; DDRD |= 0xF0
    #define SetOutp(val)      PORTC &= 0xF0; PORTC |= (val&0x0F); PORTD &= 0x0F; PORTD |= (val&0xF0)
    // PB1 Status LED
    #define InitLedPin()      PORTB &= 0xFD; DDRB |= 0x02
    #define SetLed(val)       PORTB &= 0xFD; PORTB |= (val&0x02)
    #define ToggleLed()       PORTB ^= 0x02
    // PB0 Out-CLK for latch 1
    #define InitLatch1Clk()   PORTB &= 0xFE; DDRB |= 0x01
    #define SetLatch1Clk(val) PORTB &= 0xFE; PORTB |= (val&0x01)
    // PB2 Out-CLK for latch 2
    #define InitLatch2Clk()   PORTB &= 0xFB; DDRB |= 0x04
    #define SetLatch2Clk(val) PORTB &= 0xFB; PORTB |= (val&0x04)
  #else
    // ***** Atmega8 with 2 x HC573
    // PC0-PC3, PD4-PD7
    #define InitOutpPort()    PORTC &= 0xF0; DDRC |= 0x0F; PORTD &= 0x0F; DDRD |= 0xF0
    #define SetOutp(val)      PORTC &= 0xF0; PORTC |= (val&0x0F); PORTD &= 0x0F; PORTD |= (val&0xF0)
    // PB1 Status LED
    #define InitLedPin()      PORTB &= 0xFD; DDRB |= 0x02
    #define SetLed(val)       PORTB &= 0xFD; PORTB |= (val&0x02)
    #define ToggleLed()       PORTB ^= 0x02
    // PB0 Out-CLK for latch 1
    #define InitLatch1Clk()   PORTB &= 0xFE; DDRB |= 0x01
    #define SetLatch1Clk(val) PORTB &= 0xFE; PORTB |= (val&0x01)
    // PB2 Out-CLK for latch 2
    #define InitLatch2Clk()   PORTB &= 0xFB; DDRB |= 0x04
    #define SetLatch2Clk(val) PORTB &= 0xFB; PORTB |= (val&0x04)
  #endif
  // *********************************************************************************
  // ***** ATtiny2313, ATmega8/48/88/168
  #define USB_CFG_IOPORTNAME      D
  #define USB_CFG_DMINUS_BIT      3
  #define USB_CFG_DPLUS_BIT       2

#else

  // *********************************************************************************
  #define ledInit()     PORTC |= ((1<<PC1)|(1 << PC0)); DDRC |= ((1<<PC1)|(1 << PC0))
  #define ledRedTog()   PORTC ^= (1 << PC1)
  #define ledRedOn()    PORTC &= ~(1 << PC1)
  #define ledRedOff()   PORTC |= (1 << PC1)
  #define ledGreenTog() PORTC ^= (1 << PC0)
  #define ledGreenOn()  PORTC &= ~(1 << PC0)
  #define ledGreenOff() PORTC |= (1 << PC0) 

  // *********************************************************************************
  #define USB_CFG_IOPORTNAME      B
  /* This is the port where the USB bus is connected. When you configure it to
   * "B", the registers PORTB, PINB and DDRB will be used.
   */
  #define USB_CFG_DMINUS_BIT      0
  /* This is the bit number in USB_CFG_IOPORT where the USB D- line is connected.
   * This may be any bit in the port.
   */
  #define USB_CFG_DPLUS_BIT       1
  /* This is the bit number in USB_CFG_IOPORT where the USB D+ line is connected.
   * This may be any bit in the port. Please note that D+ must also be connected
   * to interrupt pin INT0! [You can also use other interrupts, see section
   * "Optional MCU Description" below, or you can connect D- to the interrupt, as
   * it is required if you use the USB_COUNT_SOF feature. If you use D- for the
   * interrupt, the USB interrupt will also be triggered at Start-Of-Frame
   * markers every millisecond.]
   */
#endif

#define USB_CFG_CLOCK_KHZ       (F_CPU/1000)
/* Clock rate of the AVR in MHz. Legal values are 12000, 12800, 15000, 16000,
 * 16500 and 20000. The 12.8 MHz and 16.5 MHz versions of the code require no
 * crystal, they tolerate +/- 1% deviation from the nominal frequency. All
 * other rates require a precision of 2000 ppm and thus a crystal!
 * Default if not specified: 12 MHz
 */

/* ----------------------- Optional Hardware Config ------------------------ */

/* #define USB_CFG_PULLUP_IOPORTNAME   D */
/* If you connect the 1.5k pullup resistor from D- to a port pin instead of
 * V+, you can connect and disconnect the device from firmware by calling
 * the macros usbDeviceConnect() and usbDeviceDisconnect() (see usbdrv.h).
 * This constant defines the port on which the pullup resistor is connected.
 */
/* #define USB_CFG_PULLUP_BIT          4 */
/* This constant defines the bit number in USB_CFG_PULLUP_IOPORT (defined
 * above) where the 1.5k pullup resistor is connected. See description
 * above for details.
 */

/* --------------------------- Functional Range ---------------------------- */


/* Example configuration: Port C bit 2 is connected to a jumper which ties
 * this pin to GND if the boot loader is requested. Initialization allows
 * several clock cycles for the input voltage to stabilize before
 * AppCondition() samples the value.
 * We use a function for HwLowInit() for convenience and a macro for
 * AppCondition() for efficiency.
 */

#ifndef __ASSEMBLER__   /* assembler cannot parse function definitions */
#include <util/delay.h>

#ifdef HID_LIGHT_HW 
  // ****** PB4
  static inline void  HwLowInit(void)
  {
      PORTB &= ~(1 << 4); /* disable pull-up because external pull dowm */
      DDRB  &= ~(1 << 4);
      _delay_us(10);      /* wait for levels to stabilize */
  }

  #define AppCondition()   (!(PINB & (1 << 4)))  // True if jumper is not set --> low

#else

  // ****** PC2
  static inline void  HwLowInit(void)
  {
      PORTC = 1 << 2; /* activate pull-up for key */
      _delay_us(10);  /* wait for levels to stabilize */
  }

  #define AppCondition()   (PINC & (1 << 2))   // True if jumper is not set  --> low

#endif


#endif

/* ------------------------------------------------------------------------- */

#endif /* __hwconfig_h_included__ */
