/* Name: main.c
 * Project: HID-LightCtrl
 * Author: SFR frydaysoft.de
 * Creation Date: 2013-03-19
 * Tabsize: 4
 * Note: basics used from OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt)
 * This Revision: $Id: main.c 788 2010-05-30 20:54:41Z cs $
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/boot.h>
#include <string.h>
#include <util/delay.h>

static void startBootloader() __attribute__((__noreturn__));

#include "hwconfig.h"
#include "usbdrv.c"

/* ------------------------------------------------------------------------ */

#ifndef ulong
#   define ulong    unsigned long
#endif
#ifndef uint
#   define uint     unsigned int
#endif

typedef struct
{
  uchar id;
  uchar len;
} T_sReportReq;
// *********************************************************************************
static uchar          versionBuffer[7] = { 2,'V','1','.','0','0',0}; 
static uchar          cmdBuffer[7];
static uchar          bLedTest;
static T_sReportReq   sReportReq;



#define ID_REP_1  1
#define LEN_REP_1 6

#define ID_REP_2  2
#define LEN_REP_2 6

// *********************************************************************************
PROGMEM char usbHidReportDescriptor[33] = {
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)

    0x85, ID_REP_1,                //   REPORT_ID (1=simple cmd)
    0x95, LEN_REP_1,               //   REPORT_COUNT (6)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)

    0x85, ID_REP_2,                //   REPORT_ID (2=GetVersion)
    0x95, LEN_REP_2,               //   REPORT_COUNT (6)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};

/* compatibility with ATMega88 and other new devices: */
#ifndef TCCR0
#define TCCR0   TCCR0B
#endif
#ifndef GICR
#define GICR    MCUCR
#endif

// *********************************************************************************
static void startBootloader()
{
    DBG1(0x01, 0, 0);
    cli();
    /* activate pull-ups except on USB lines */
    USB_CFG_IOPORT   = (uchar)~((1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT));
    /* all pins input except USB (-> USB reset) */
#ifdef USB_CFG_PULLUP_IOPORT    /* use usbDeviceConnect()/usbDeviceDisconnect() if available */
    USBDDR    = 0;    /* we do RESET by deactivating pullup */
    usbDeviceDisconnect();
#else
    USBDDR    = (1<<USB_CFG_DMINUS_BIT)|(1<<USB_CFG_DPLUS_BIT);
#endif
    wdt_enable(WDTO_1S);
    while(1) {;}
}


// *********************************************************************************
uchar   usbFunctionSetup(uchar data[8])
{
    uchar  ret = 0;

    // eg: 21,09,01,03,00,00,07 for HidD_SetFeature("01 ......")
    // data[0]   = rq->bmRequestType
    // data[1]   = rq->bRequest
    // data[2/3] = rq->wValue
    // data[4/5] = rq->wIndex
    // data[6/7] = rq->wLength

    // ***** rq->bRequest
    switch(data[1]) 
    {
      case USBRQ_HID_SET_REPORT:  // 0x09
        sReportReq.id  = data[2]; // report ID
        sReportReq.len = data[6]; // report length
        ret = USB_NO_MSG;
        break;
      case USBRQ_HID_GET_REPORT:  // 0x01
        switch(data[2])           // report ID
        {
          case ID_REP_1:
            usbMsgPtr = cmdBuffer;
            ret = (LEN_REP_1+1);
            break;
          case ID_REP_2:
            usbMsgPtr = versionBuffer;
            ret = (LEN_REP_2+1);
            break;
          default:
            break;
        }
      default:
        break;
    }
    return ret;
}


// *********************************************************************************
uchar usbFunctionWrite(uchar *data, uchar len)
{
  if (sReportReq.len)
  {
    if (sReportReq.len == len)
    {
      switch(sReportReq.id)
      { 
        // ***** simple cmd, max. 5 bytes left
        case ID_REP_1:
          memcpy(&cmdBuffer[0], &data[0], LEN_REP_1);
          switch(data[1])
          { // ***** set lights
            case 0:
              SetOutp(data[2]);
              //SetOutp2(data[3]);
              break;
            // ***** start/stop LED test
            case 1:
              bLedTest = data[2];
              break;
            default:
              break;
          }
          break;
        // ***** get version has no write access
        case ID_REP_2:
        default:
          break;
      }
    }
  }
  sReportReq.id  = 0;
  sReportReq.len = 0;
  return 0x80; /* != 0 if last block received */
}


#define USE_T1
// *********************************************************************************
void TimerInit(void)
{ 
#ifdef USE_T1
  // ***** stop Timer1
  TCCR1A = 0x00;
  TCCR1B = 0x00;   
  // ***** set Output Compare Register
  OCR1A  = 18750;  // 1200000 / 64 * 0,1 = 18750
  TIFR   = (1<<OCF1A); 
  TCCR1B = 0x0B;   // clear Timer1 on CTC, prescaler 64
#else
#endif
}


// *********************************************************************************
uchar timerPoll()
{
#ifdef USE_T1
  if (TIFR & (1<<OCF1A))
  {
    TIFR  |= (1<<OCF1A); 
	  return 1;
  }
  return 0;
#else
  return 1;
#endif
}


// *********************************************************************************
static void initForUsbConnectivity(void)
{
    uchar   i = 0;

#if F_CPU == 12800000
    TCCR0 = 3;          /* 1/64 prescaler */
#endif
    usbInit();
    /* enforce USB re-enumerate: */
    usbDeviceDisconnect();  /* do this while interrupts are disabled */
    do{             /* fake USB disconnect for > 250 ms */
        wdt_reset();
        _delay_ms(1);
    }while(--i);
    usbDeviceConnect();
    sei();
}


// *********************************************************************************
void hardwareInit(void)
{
    uchar   i;

    usbDeviceDisconnect();  // enforce re-enumeration, do this while interrupts are disabled!
    i = 0;
    while(--i)
    { // ***** fake USB disconnect for > 250 ms
      wdt_reset();
      _delay_ms(1);
    }
    usbDeviceConnect();

    #ifdef HID_LIGHT_HW
    SetOutp(0);
    InitLedPin();
    InitLatch1Clk();
    InitLatch2Clk();
    InitOutpPort();
    #else
    ledInit();
    #endif
}


// *********************************************************************************
int __attribute__((noreturn)) main(void)
{
    uchar   steps=0, leds = 1;

    wdt_enable(WDTO_1S); 
    odDebugInit();
    DBG1(0x00, 0, 0);
    // ***** initialize hardware
    HwLowInit();
    hardwareInit(); 
    sReportReq.len = 0;

    usbInit();

    sei();
    DBG1(0x01, 0, 0);
    TimerInit();

    //GICR = (1 << IVCE);     // enable change of interrupt vectors
    //GICR = (0 << IVSEL);    // move interrupts to application flash section

    bLedTest = 0;
    steps    = 0; 
    leds     = 1;

    // ***** main event loop 
    do
    { 
      wdt_reset();
      usbPoll();
      if (timerPoll())
      {
        if (++steps > 10)
        {
          if (bLedTest)
          {
          #ifdef HID_LIGHT_HW
            SetOutp(leds);
            leds <<= 1;
            if (!leds) leds=1;
          #else
            ledRedTog();
          #endif
          }
          steps = 0;
        }
      } 
    }while(AppCondition());

    startBootloader();
}

